/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDServerError.h"


@implementation MPDServerError

static MPDServerError *sMPDServerErrorManager = nil;

// Singleton
+ (MPDServerError *)sMPDServerError {
    if (sMPDServerErrorManager == nil)
        sMPDServerErrorManager = [[super allocWithZone:NULL] init];
    
    return sMPDServerErrorManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDServerError];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    return self;
}


- (void)send:(struct mpd_connection *)mpdConnection {
    assert(mpd_connection_get_error(mpdConnection) != MPD_ERROR_SUCCESS);
    
    NSString *error =  [NSString stringWithFormat:@"%s", mpd_connection_get_error_message(mpdConnection)];
    NSString *description = @"pas d'information complémentaire";
    
    if ([error isEqualToString:@"Connection closed by the server"]) {
        
        error = @"Connexion fermée par le serveur";
        description = @"Le serveur MPD ne répond plus.\n\n Vérifiez qu'il n'a pas été tué et que la connexion n'a pas de problème.";
        
    } else if ([error isEqualToString:@"Connection reset by peer"]) {
        
        error = @"Réinitialisation de la connexion";
        description = @"1. Vérifiez que le serveur MPD est joignable et le service toujours actif.\n\n2. Vérifiez que la connexion réseau est fiable.";
        
    } else if ([error isEqualToString:@"Connection refused"]) {
        
        error = @"Connexion rejetée";
        description = @"1. Vérifiez que le serveur est joignable et que MPD est actif.\n2. Vérifiez qu'un firewall ne bloque pas la connexion.\n3. Vérifiez votre connexion réseau.";
    }
    
    
    if (error != nil) {
        MPDError *mpderror = [[MPDError alloc] init];
        [mpderror setError:error];
        [mpderror setErrorDescription:description];
        [self.delegate onError:mpderror];
    }
}

@end
