/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDAlbum.h"
#import "MPDUtils.h"
#import "MPDSocket.h"
#import "MPDServerError.h"
#import "Song.h"


@implementation MPDAlbum


- (id)init {
    self = [super init];
    return self;
}

- (NSArray *)getAlbumsList {
    if (![[MPDSocket sMPDSocket] isConnected])
        return nil;
        
    NSMutableArray *pathList = [NSMutableArray array];
    
    mpd_send_list_all([[MPDSocket sMPDSocket] getConnection], "/");
        
    struct mpd_entity * entity;
        
    // get all direcrectory contain song
    [self getAllDirectory:&entity pathList:pathList];
        
    return [pathList deleteDuplicateAlbum:[[MPDSocket sMPDSocket] getConnection]].sortAlbum;
}

- (void)getAllDirectory:(struct mpd_entity **)entity pathList:(NSMutableArray *)pathList {
    
    while ((*entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
        const struct mpd_song *song;
        
        if (mpd_entity_get_type(*entity) == MPD_ENTITY_TYPE_SONG) {
            song = mpd_entity_get_song(*entity);
            
            NSString *list = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
            
            // Get path only
            NSString * plop = [list stringByDeletingLastPathComponent];
            
            [pathList addObject:plop];
        }
        
        mpd_entity_free(*entity);
    }
}


- (NSArray *)getAlbumContent:(Album *)aAlbum {
    
    if (![[MPDSocket sMPDSocket] isConnected])
        return nil;
        
    NSMutableArray *albumContent = [NSMutableArray array];
    NSString *p = aAlbum.path;
    const char *path = [p UTF8String];
    struct mpd_entity * entity;
        
    if (!mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], path)) {
        [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
        return nil;
    }
        
    [self getAllSongs:albumContent entity:&entity];
    
    return albumContent;
}

- (void)getAllSongs:(NSMutableArray *)albumContent entity:(struct mpd_entity **)entity {
    
    while ((*entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
        const struct mpd_song *mpdsong;
        
        if (mpd_entity_get_type(*entity) == MPD_ENTITY_TYPE_SONG) {
            mpdsong = mpd_entity_get_song(*entity);
            Song *song = [[Song alloc] initWithMPDSong:mpdsong];
            [albumContent addObject:song];
        }
        
        mpd_entity_free(*entity);
    }
}


@end
