/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDQueue.h"
#import "MPDSocket.h"
#import "MPDUtils.h"
#import "MPDServerError.h"
#import "Song.h"


@implementation MPDQueue


- (id)init {
    self = [super init];
    return self;
}


- (NSArray *)getQueue {
    
    if (![[MPDSocket sMPDSocket] isConnected])
        return nil;
        
    struct mpd_status * status;
    mpd_send_status([[MPDSocket sMPDSocket] getConnection]);
    status = mpd_recv_status([[MPDSocket sMPDSocket] getConnection]);
        
    if (status == NULL) {
        [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
        return nil;
    } else {
        int queueLenght = mpd_status_get_queue_length(status);
        return [self getSongs:queueLenght];
    }
}


- (NSArray *)getSongs:(int)queueLenght {
    NSMutableArray *queue = [NSMutableArray array];
    struct mpd_song *mpdsong;
    unsigned int i = 0;
    
    while (i < queueLenght) {
        
        mpdsong = mpd_run_get_queue_song_pos([[MPDSocket sMPDSocket] getConnection], i);
        i++;
        
        Song *song = [[Song alloc] initWithMPDSong:mpdsong];
        
        [queue addObject:song];
        mpd_song_free(mpdsong);
    }
    
    return queue;
}


- (void)addToQueue:(NSArray *)array {
    
    if (![[MPDSocket sMPDSocket] isConnected])
        return;
        
    for (Song *song in array) {
        const char *file = [song.fullPath UTF8String];
        mpd_run_add([[MPDSocket sMPDSocket] getConnection], file);
    }
}


- (void)removeInQueue:(NSArray *)array {
    
    if (![[MPDSocket sMPDSocket] isConnected])
        return;
    
    unsigned int i = 0;
    for (NSString *position in array) {
        mpd_run_delete([[MPDSocket sMPDSocket] getConnection], [position intValue] - i);
        i++;
    }
}


- (void)saveQueueAtPlaylist:(NSString *)playlist {
    if (![[MPDSocket sMPDSocket] isConnected])
        return;
    
    const char *list = [playlist UTF8String];
    mpd_run_save([[MPDSocket sMPDSocket] getConnection], list);
}


@end
