/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#import "MPDUtils.h"
#import "Album.h"


@implementation NSString (OGSongValue)

+ (GetTag *)song:(const struct mpd_song *)song tag:(enum mpd_tag_type)tagType {
    unsigned i = 0;
    const char *value;
    
    while ((value = mpd_song_get_tag(song, tagType, i++)) != NULL)
        return [NSString stringWithUTF8String:value];
    
    return nil;
}

@end


@implementation NSArray (OGSort)

- (NSArray *)sortAlbum {
    
    NSArray *sortedArray = [self sortedArrayUsingComparator:^(Album* lhn, Album* rhn) {
    
        NSString *leftArtist = [self formatName:lhn.artist];
        NSString *rightArtist = [self formatName:rhn.artist];
        
        if ([leftArtist isEqualToString:rightArtist]) {
            
            int leftYear = lhn.date;
            int rightYear = rhn.date;
            
            NSString *leftYearString = [NSString stringWithFormat:@"%d",leftYear];
            NSString *rightYearString = [NSString stringWithFormat:@"%d",rightYear];
            
            if (leftYear == rightYear) {
                // Tri en fonction du nom du chemin si identique
                NSString *leftPath =  lhn.path;
                NSString *rightPath = rhn.path;
                
                return [leftPath compare:rightPath options:NSCaseInsensitiveSearch];
            }
            
            return [leftYearString compare:rightYearString options:NSCaseInsensitiveSearch];
            
        } else {
            
            return [leftArtist compare:rightArtist options:NSCaseInsensitiveSearch];
        }
        
    }];
    
    return sortedArray;
}

- (NSArray *)sortList {
    
    NSArray *sortedArray = [self sortedArrayUsingComparator:^(NSString* lhn, NSString* rhn) {
        NSString *leftStream = lhn;
        NSString *rightStream = rhn;
        
        return [leftStream compare:rightStream options:NSCaseInsensitiveSearch];
    }];
    
    return sortedArray;
}


- (NSString *)formatName:(NSString *)aString {
    
    if([aString hasPrefix:@"The "])
        return [aString substringFromIndex:4];
    else
        return aString;
}

@end


@implementation NSArray (OGDeleteDuplicate)

- (NSArray *)deleteDuplicateAlbum:(struct mpd_connection*)connection {
    
    NSMutableArray *albumList = [NSMutableArray array];
    NSDictionary *uniq = [NSDictionary dictionaryWithObjects:self forKeys:self];

    struct mpd_entity * entity;
    
    for (NSString *path in uniq) {
        
        const char *h = [path UTF8String];
        
        mpd_send_list_meta(connection, h);
        
        while ((entity = mpd_recv_entity(connection)) != NULL) {
            const struct mpd_song *song;
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
                song = mpd_entity_get_song(entity);
                
                NSString *artist = [GetTag song:song tag:MPD_TAG_ALBUM_ARTIST];
                
                if (artist == nil)
                    artist = [GetTag song:song tag:MPD_TAG_ARTIST];
                
                NSString *alb = [GetTag song:song tag:MPD_TAG_ALBUM];
                BOOL mrBool = NO;
                
                for (Album *s in albumList) {
                    if ([alb isEqualToString:s.albumName] && [artist isEqualToString:s.artist])
                        mrBool = YES;
                }
                
                if (!mrBool) {
                    Album *model = [[Album alloc]init];
                    [model setArtist:artist];
                    [model setAlbumName:alb];
                    [model setGenre:[GetTag song:song tag:MPD_TAG_GENRE]];
                    [model setDate:[[GetTag song:song tag:MPD_TAG_DATE] intValue]];
                    [model setPath:path];
                    [albumList addObject:model];
                }
            }
    
            mpd_entity_free(entity);
        }
    }
    return albumList;
}


@end
