/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDCache.h"


@implementation MPDCache

MPDStatus *mpdStatus = nil;

static MPDCache *sMPDCacheManager = nil;

NSString *actualTrack = nil;

int statePlayer = -1;
int currentPosition = 0;
unsigned currentVolume = 0;

// delai timer / 2 pour la progressbar
BOOL progressBarloop = NO;

// Singleton
+ (MPDCache *)sMPDCache {
    if (sMPDCacheManager == nil)
        sMPDCacheManager = [[super allocWithZone:NULL] init];
    
    return sMPDCacheManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDCache];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id) init {
    self = [super init];
    return self;
}

- (void)setStatus:(MPDStatus *)aMPDStatus {
    mpdStatus = aMPDStatus;
    
    progressBarloop = !progressBarloop;
    
    if (mpdStatus == nil)
        return;
    
    NSString *track = [NSString stringWithFormat:@"%@%@%d", mpdStatus.album, mpdStatus.title, mpdStatus.titlePosition];
    
    if (progressBarloop)
        [self.delegate onProgressChange:mpdStatus];
    
    
    if (![actualTrack isEqualToString:track]) {
        [self.delegate onTrackChange:mpdStatus];
        [self.delegateQueue onTrackChange:mpdStatus.titlePosition];
        
        currentPosition = mpdStatus.titlePosition;
        actualTrack = track;
    }
    
    if (mpdStatus.volume != currentVolume) {
        [self.delegate onVolumeChange:mpdStatus.volume];
        currentVolume = mpdStatus.volume;
    }
    
    if (statePlayer != mpdStatus.state) {
        [self.delegateQueue onStateChange:mpdStatus.state position:mpdStatus.titlePosition];
        [self.delegate onStateChange:mpdStatus.state];
        statePlayer = mpdStatus.state;
    }
    
}

- (MPDStatus *)getStatus {
    return mpdStatus;
}

- (void)resetStatus {
    actualTrack = nil;
    statePlayer = -1;
}

- (int) getPosition {
    return currentPosition;
}

- (void)dbUpdated {
    [self.delegateAlbum onDbUpdate];
    [self.delegate onAlbumUpdateTrigger:@"dbUpdateFinish"];
}

- (void)albumUpdateTrigger:(NSString *)msg {
    [self.delegate onAlbumUpdateTrigger:msg];
}

- (void)streamUpdateTrigger {
    [self.delegateStream onStreamChange];
}

- (void)radioLinkTrigger:(Radio *)radio {
    [self.delegateRadio onRadioLinkClick:radio];
}


@end
