/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <include/mpd/status.h>
#include <SDImageCache.h>
#import "MainActivity.h"
#import "Model/Stream.h"
#import "MPDService/MPDCommand.h"
#import "MPDService/Helper/MPDPlayLists.h"
#import "MPDService/Helper/MPDQueue.h"
#import "MPDService/Helper/MPDStream.h"
#import "Setting/Preferences.h"
#import "Sidebar/SidebarItem.h"
#import "Views/Playlist/CurrentPlaylisyView.h"
#import "Views/Dialog/ConfirmDialog.h"
#import "Views/Dialog/SetNameDialog.h"
#import "Views/Stream/StreamEdit.h"


@interface MainActivity ()

@property (weak) IBOutlet NSWindow *window;
@end

@implementation MainActivity

Preferences *preferences;
MPDCommand *mpd;
ConfirmDialog *confirmDialog;
SetNameDialog *setNameDialog;
StreamEdit *newStream;

NSViewController *currentContentViewController;
NSMutableDictionary *sideBarItem;
NSArray *sideBarTopLevelItem;
NSString *vueActuelle = nil;
NSString *playlistAddDelete;

BOOL dialogLocked = NO;
BOOL dbUpdating = NO;

MPDPlayLists *mpdPlayLists;

- (void)awakeFromNib {
    [sideBar selectRowIndexes:[NSIndexSet indexSetWithIndex:1] byExtendingSelection:NO];
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    
    [MPDServerError sMPDServerError].delegate = self;
    [MPDCache sMPDCache].delegate = self;
    mpd = [MPDCommand sMPDCommand];
    
    mpdPlayLists = [[MPDPlayLists alloc] init];
    
    sideBarTopLevelItem = @[@"Source", @"Playlist"];
    
    sideBarItem = [NSMutableDictionary new];
    sideBarItem[@"Source"] = @[@"Library", @"Queue", @"Stream", @"Radio"];
    
    NSArray *playlistList = mpdPlayLists.getPlaylists;
    
    if (playlistList != nil)
        sideBarItem[@"Playlist"] = playlistList;
    
    [sideBar reloadData];
    [sideBar expandItem:nil expandChildren:YES];
    
    [[MPDCache sMPDCache] setMainWindow:[self window]];
    
    [self playlistListener];
}


- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication {
    return YES;
}


/* ********************************** *
 *              SideBar               *
 * ********************************** */
- (void)setContentViewToName:(NSString *)name {
    
    if (dbUpdating) // évite de crasher
        return;
    
    vueActuelle = name;
        
    [addRadio setHidden:![name isEqualToString:@"Stream"]];
        
    // Utilise le même xib pour toutes les playlistes
    if (![name isEqualToString:@"Library"] && ![name isEqualToString:@"Queue"] && ![name isEqualToString:@"Stream"] && ![name isEqualToString:@"Radio"]) {
            
        CurrentPlaylisyView.currentPlaylist = name;
        name = @"Playlist";
    }
        
    if (currentContentViewController)
        [[currentContentViewController view] removeFromSuperview];
        
    currentContentViewController = [[NSViewController alloc] initWithNibName:name bundle:nil];
    NSView *view = [currentContentViewController view];
    view.frame = masterView.bounds;
    [view setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    [masterView addSubview:view];
}


- (void)outlineViewSelectionDidChange:(NSNotification *)notification {
    if ([sideBar selectedRow] != -1) {
        NSString *item = [sideBar itemAtRow:[sideBar selectedRow]];
        [self setContentViewToName:item];
    }
}

- (NSArray *)childrenForItem:(id)item {
    NSArray *children;
    
    if (item == nil)
        children = sideBarTopLevelItem;
    else
        children = sideBarItem[item];

    return children;
}

- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item {
    return [self childrenForItem:item][index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item {
    return [outlineView parentForItem:item] == nil;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item {
    
    if (![outlineView parentForItem:item])
        return NO;
    else
        return YES;
}

- (NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item {
    return [[self childrenForItem:item] count];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isGroupItem:(id)item {
    return [sideBarTopLevelItem containsObject:item];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldShowOutlineCellForItem:(id)item {
    return ![item isEqualToString:@"Source"];
}

- (NSView *)outlineView:(NSOutlineView *)outlineView viewForTableColumn:(NSTableColumn *)tableColumn item:(id)item {
    
    if ([sideBarTopLevelItem containsObject:item]) {
        
        NSTextField *result = [outlineView makeViewWithIdentifier:@"HeaderTextField" owner:self];
        
        [result setStringValue:item];
       
        return result;
    }
    
        
    SidebarItem *result = [outlineView makeViewWithIdentifier:@"MainCell" owner:self];
    result.textField.stringValue = item;
    [result.button setHidden:YES];
        
    if ([item isEqualToString:@"Library"]) {
            
        result.textField.stringValue = @"Albums";
        result.imageView.image = [NSImage imageNamed:@"sidebar_album"];
            
        NSString *count = [NSString stringWithFormat:@"%d",(int)[[[MPDCache sMPDCache] currentAlbumList] count]];
        [result.button setTitle:count];
        [result.button sizeToFit];
        [result.button setHidden:NO];
            
    } else if ([item isEqualToString:@"Queue"]) {
            
        result.textField.stringValue = @"Lecture en cours";
        result.imageView.image = [NSImage imageNamed:@"sidebar_queue"];
            
    } else if ([item  isEqualToString:@"Stream"]) {
            
        result.imageView.image = [NSImage imageNamed:@"sidebar_stream"];
            
    } else if ([item  isEqualToString:@"Radio"]) {
            
        result.imageView.image = [NSImage imageNamed:@"sidebar_radio"];
            
    } else {
            
        result.imageView.image = [NSImage imageNamed:@"sidebar_playlist"];
    }
        
    return result;
}


// La sidebar n'est plus élastique lors de redimensionnement
-(void)splitView:(NSSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize {
    
    CGFloat dividerThickness = [sender dividerThickness];
    NSRect leftRect  = [[sender subviews][0] frame];
    NSRect rightRect = [[sender subviews][1] frame];
    NSRect newFrame  = [sender frame];
    
    leftRect.size.height = newFrame.size.height;
    leftRect.origin = NSMakePoint(0, 0);
    rightRect.size.width = newFrame.size.width - leftRect.size.width - dividerThickness;
    rightRect.size.height = newFrame.size.height;
    rightRect.origin.x = leftRect.size.width + dividerThickness;
    
    [[sender subviews][0] setFrame:leftRect];
    [[sender subviews][1] setFrame:rightRect];
}


- (void)updateSidebar {
    NSArray *playlistList = mpdPlayLists.getPlaylists;
    
    if (playlistList != nil)
        sideBarItem[@"Playlist"] = playlistList;
    
    [sideBar reloadData];
}


/* ********************************** *
 *           Menu contextuel          *
 * ********************************** */
- (void)menuNeedsUpdate:(NSMenu *)menu {
    
    NSInteger clickedrow = [sideBar clickedRow];
    NSString *title1;
    NSMenuItem *item1;
    
    [menu removeAllItems];
    
    switch (clickedrow) {
        case 2: // lecture en cours
            title1 = [NSString stringWithFormat:@"Enregistrer comme playlist"];
            item1 = [[NSMenuItem alloc] initWithTitle:title1 action:@selector(saveAsPlaylist:) keyEquivalent:@""];
            [menu addItem:item1];
            break;
            
        case -1: // hors item
            title1 = [NSString stringWithFormat:@"Nouvelle playlist"];
            item1 = [[NSMenuItem alloc] initWithTitle:title1 action:@selector(newPlaylist:) keyEquivalent:@""];
            [menu addItem:item1];
            break;
            
        default:
            
            if (clickedrow < 6) // La première playlist = 6ème position
                break;
                
            NSArray *sideArray = sideBarItem[@"Playlist"];
            NSString *name = sideArray[clickedrow - 6];
            playlistAddDelete = name;
            
            title1 = [NSString stringWithFormat:@"Renommer"];
            item1 = [[NSMenuItem alloc] initWithTitle:title1 action:@selector(renamePlaylist:) keyEquivalent:@""];
                
            NSString *title2 = [NSString stringWithFormat:@"Supprimer"];
            NSMenuItem *item2 = [[NSMenuItem alloc] initWithTitle:title2 action:@selector(deletePlaylist:) keyEquivalent:@""];
        
            [menu addItem:item1];
            [menu addItem:item2];
            
            break;
    }
    
}

/* **************** Menu contextuel ****************** */


-(void)deletePlaylist:(id)sender {
    
    if (dialogLocked)
        return;
    
    NSString *text = [NSString stringWithFormat:@"Confirmer la suppression de la playlist: %@", playlistAddDelete];
        
    NSMutableArray *marray = [NSMutableArray array];
        
    [marray addObject:@"Supprimer une playlist"];
    [marray addObject:text];
    [marray addObject:@"playlistBoxClose"];
        
    confirmDialog = [[ConfirmDialog alloc] init:marray];
    [confirmDialog showWindow:self];
    dialogLocked = YES;
}

- (void)newPlaylist:(id)sender {
        [self nsPlaylist:@"Nouvelle playlist" action:@"new"];
}

- (void)renamePlaylist:(id)sender {
    [self nsPlaylist:@"Renommer la playlist" action:@"rename"];
}

- (void)saveAsPlaylist:(id)sender {
    [self nsPlaylist:@"Enregistrer comme playlist" action:@"save"];
}

- (void)nsPlaylist:(NSString *)titre action:(NSString *)action {
    
    if (dialogLocked)
        return;
    
    NSMutableArray *marray = [NSMutableArray array];
        
    [marray addObject:titre];
    [marray addObject:@"playlistBoxClose"];
    [marray addObject:action];
        
    setNameDialog = [[SetNameDialog alloc] init:marray];
    [setNameDialog showWindow:self];
    dialogLocked = YES;
}

/* ***************  SideBar  *************** */

- (void)playlistListener {
    
    [[NSNotificationCenter defaultCenter] addObserverForName:@"playlistBoxClose" object:nil queue:nil usingBlock:^(NSNotification *note) {
        
        NSDictionary *userInfo = note.userInfo;
        NSString *what = (NSString *)userInfo[@"close"];
        NSString *value = (NSString *)userInfo[@"name"];
        
        if ([what isEqualToString:@"delete"]) {
            [mpdPlayLists deletePlaylist:playlistAddDelete];
        } else if ([what isEqualToString:@"save"]) {
            MPDQueue *mpdQueue = [[MPDQueue alloc] init];
            [mpdQueue saveQueueAtPlaylist:value];
        } else if ([what isEqualToString:@"new"]) {
            [mpdPlayLists newPlaylist:value];
        } else if ([what isEqualToString:@"rename"]) {
            [mpdPlayLists renamePlaylist:playlistAddDelete newName:value];
        }
        
        dialogLocked = NO;
        
        [self updateSidebar];
    }];
}


- (void)onStateChange:(int)state {
    
    if (state == MPD_STATE_PLAY)
        [playImg setImage:[NSImage imageNamed:@"pause_button"]];
    else
        [playImg setImage:[NSImage imageNamed:@"play_button"]];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"statusChanged" object:nil];
}


- (void)onVolumeChange:(unsigned int)volume {
    volumeSlider.intValue = volume;
}


- (void)onProgressChange:(MPDStatus *)status {
    
    progressTrack.floatValue = status.elapsedTime/1000.0f;
    if (status.elapsedTime > 0) {
        int elapsedMinutes = (status.elapsedTime/1000) / 60;
        int elapsedSeconds = (status.elapsedTime/1000) % 60;
        NSString *elapsed = [NSString stringWithFormat:@"%2d:%02d",elapsedMinutes, elapsedSeconds];
        [elapsedTime setStringValue:elapsed];
    }
}


- (void)onTrackChange:(MPDStatus *)status {
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"statusChanged" object:nil userInfo:nil];
    
    progressTrack.maxValue = status.totalTime;
    seekToBar.maxValue = status.totalTime;
    
    [self setTotalTime:status.totalTime];
    [self setArtistAndAlbum:status];
    [self setSongTitle:status];
}


- (void)onAlbumUpdateTrigger:(NSString *)msg {
    
    if ([msg isEqualToString:@"dbUpdateFinish"]) {
        dbUpdating = NO;
        [self onTrackChange:[[MPDCache sMPDCache] getStatus]];
        [sideBar reloadData];
    } else {
        [currentArtistAlbum setStringValue:@" "];
        [currentSongTitle setStringValue:msg];
    }
}


- (void)onError:(MPDError *)error {
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Ok"];
    [alert setMessageText:error.error];
    [alert setInformativeText:error.errorDescription];
    [alert setAlertStyle:NSAlertStyleWarning];
    [alert setIcon:[NSImage imageNamed:@"mpd_server_error"]];
    [alert beginSheetModalForWindow:[self window] completionHandler:^(NSModalResponse returnCode) {
        NSLog(@"%@", error.errorDescription);
    }];
}
- (void)setTotalTime:(int)aTotalTime {
    
    if (aTotalTime > 0) {
        int totalMinutes = aTotalTime / 60;
        int totalSeconds = aTotalTime % 60;
        NSString *total = [NSString stringWithFormat:@"%2d:%02d",totalMinutes, totalSeconds];
        
        [songLenght setStringValue:total];
    }
}

- (void)setArtistAndAlbum:(MPDStatus *)status {
    NSString *at;
    
    if (status.artist == nil && status.album != nil)
        at = [NSString stringWithFormat:@"%@", status.album];
    else if (status.artist != nil && status.album == nil)
        at = [NSString stringWithFormat:@"%@", status.artist];
    else if (status.artist == nil && status.album == nil)
        at = [NSString stringWithFormat:@""];
    else
        at = [NSString stringWithFormat:@"%@ - %@", status.artist, status.album];
    
    [currentArtistAlbum setStringValue:at];
}

- (void)setSongTitle:(MPDStatus *)status {
    NSString *title = @" ";
    
    if ( status.state == MPD_STATE_STOP) {
        title = @"Stop";
        [playImg setImage:[NSImage imageNamed:@"play_button"]];
    } else if (status.title == nil && (status.state == MPD_STATE_PLAY || status.state == MPD_STATE_PAUSE)) {
        title = @"Non renseigné";
    } else if (status.state == MPD_STATE_PLAY || status.state == MPD_STATE_PAUSE) {
        
        title = status.title;
    }
    
    NSString *path = status.path;
    
    // affichage du titre si un stream
    if ([[path substringToIndex:4] isEqualToString:@"http"]) {
        MPDStream *mpdStream = [[MPDStream alloc] init];
        NSArray *streamArray =  [mpdStream getStreamList];
        
        for (Stream *stream in streamArray) {
            NSString *url = stream.url;
            
            if ([url isEqualToString:status.path])
                title = stream.name;
        }
    }
    
    [currentSongTitle setStringValue:title];
}




- (IBAction)preferences:(id)sender {
    preferences = [[Preferences alloc] init];
    [preferences showWindow:self];
}

- (IBAction)previousButton:(id)sender {
    [mpd previous];
}

- (IBAction)nextButton:(id)sender {
    [mpd next];
}

- (IBAction)playButton:(id)sender {
    [mpd play];
}

- (IBAction)stopButton:(id)sender {
    [mpd stop];
}

- (IBAction)updateDataBase:(id)sender {
    dbUpdating = YES;
    [mpd updateDB];
}

- (IBAction)seekToPosition:(id)sender {
    
    NSString *path = [[MPDCache sMPDCache] getStatus].path;
    
    if (![[path substringToIndex:4] isEqualToString:@"http"]) {
        [mpd seekPosition:seekToBar.floatValue];
        progressTrack.floatValue = seekToBar.floatValue;
    }
}

- (IBAction)connecter:(id)sender {
    [[MPDCommand sMPDCommand] connect];
}

- (IBAction)deconnecter:(id)sender {
    [[MPDCommand sMPDCommand] disconnect];
}

- (IBAction)ajouterRadio:(id)sender {
    newStream = [[StreamEdit alloc] init:nil];
    [newStream showWindow:self];
}

- (IBAction)addNewPlaylist:(id)sender {
    [self newPlaylist:sender];
}

- (IBAction)saveCurrentPlayingList:(id)sender {
    [self saveAsPlaylist:nil];
}

- (IBAction)purgeCoverCache:(id)sender {
    SDImageCache *imageCache = [SDImageCache sharedImageCache];
    [imageCache clearMemory];
    [imageCache clearDiskOnCompletion:^(){}];
}

- (IBAction)volume:(id)sender {
    [mpd setVolume:volumeSlider.intValue];
}


@end
