/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "Song.h"

@implementation Song

- (id)initWithMPDSong:(const struct mpd_song *)mpdsong {
    self = [super init];
    
    if (!self)
        return nil;
    
    NSString *songPath = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(mpdsong) encoding:NSUTF8StringEncoding]];
    
    NSString *ext = [songPath pathExtension];
    
    const struct mpd_audio_format *format = mpd_song_get_audio_format(mpdsong);
    
    int bit = 0;
    if ([ext isEqualToString:@"dsf"] || [ext isEqualToString:@"dff"])
        bit = 1;
    else if (format != NULL)
        bit = format->bits;
    
    _artist = [GetTag song:mpdsong tag:MPD_TAG_ARTIST];
    _albumArtist = [GetTag song:mpdsong tag:MPD_TAG_ALBUM_ARTIST];
    _composer = [GetTag song:mpdsong tag:MPD_TAG_COMPOSER];
    _album = [GetTag song:mpdsong tag:MPD_TAG_ALBUM];
    _title = [GetTag song:mpdsong tag:MPD_TAG_TITLE];
    _fullPath = songPath;
    _genre = [GetTag song:mpdsong tag:MPD_TAG_GENRE];
    _comment = [GetTag song:mpdsong tag:MPD_TAG_COMMENT];
    _extension = ext;
    _track = [[GetTag song:mpdsong tag:MPD_TAG_TRACK] intValue];
    _diskNb = [[GetTag song:mpdsong tag:MPD_TAG_DISC] intValue];
    _date = [[GetTag song:mpdsong tag:MPD_TAG_DATE] intValue];
    _duration = mpd_song_get_duration(mpdsong);
    _bits = bit;
    
    return self;
}


@end
