/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "Preferences.h"
#import "Setting.h"


Setting *settingPref;

NSString *host;
NSString *port;
NSString *user;
NSString *pass;
NSString *hostHTTP;
NSString *argsPHP;


@implementation Preferences

- (instancetype)init {
    
    self = [super initWithWindowNibName:@"Preferences"];
    
    settingPref = [[Setting alloc] init];
    
    host = settingPref.host;
    port = [NSString stringWithFormat:@"%d", settingPref.port];
    user = settingPref.user;
    pass = settingPref.pass;
    hostHTTP = settingPref.http;
    argsPHP = settingPref.php;
    
    return self;
}


- (void)awakeFromNib {
    [serverField setStringValue:host];
    [portField setStringValue:port];
    
    if (user != nil)
        [usernameField setStringValue:user];
    
    if (pass != nil)
        [passwordField setStringValue:pass];
    
    if (hostHTTP != nil) {
        [checkButtonHTTP setState:NSControlStateValueOff];
        [serverHTTPField setEnabled:YES];
        [serverHTTPField setStringValue:hostHTTP];
    }
    
    if (argsPHP != nil) {
        [checkButtonPHP setState:NSControlStateValueOn];
        [argsPHPField setEnabled:YES];
        [argsPHPField setStringValue:argsPHP];
    }
    
    [self.prefwindow makeKeyAndOrderFront:nil];
    [self.prefwindow setLevel:NSStatusWindowLevel];
}


- (IBAction)save:(id)sender {
    
    host = [serverField stringValue];
    port = [portField stringValue];
    user = [usernameField stringValue];
    pass = [passwordField stringValue];
    
    hostHTTP = [serverHTTPField stringValue];
    argsPHP = [argsPHPField stringValue];
    
    if (![self checkValidityValue])
        return;
    
    [self writePreferences];
    [self.prefwindow close];
}


- (BOOL)checkValidityValue {
    
    BOOL mrBool = YES;
    
    if ([settingPref isValidIP:host]) {
        labelIP.textColor   = [NSColor labelColor];
    } else {
        labelIP.textColor = errorColor;
        mrBool = NO;
    }
    
    // Vérifie qu'il n'y a que des chiffres
    NSCharacterSet *numberSet = [NSCharacterSet decimalDigitCharacterSet];
    NSString *trimmedString = [port stringByTrimmingCharactersInSet:numberSet];
    
    if (port.length > 0 && trimmedString.length == 0 && [port integerValue] < 65536) {
        labelPort.textColor = [NSColor labelColor];
    } else {
        labelPort.textColor = errorColor;
        mrBool = NO;
    }
    
    return mrBool;
}


- (void)writePreferences {
    
    [settingPref setHost:host];
    [settingPref setPort:[port floatValue]];
    
    if (user.length > 0)
        [settingPref setUser:user];
    else
        [settingPref unsetUser];
    
    if (pass.length > 0)
        [settingPref setPass:pass];
    else
        [settingPref unsetPass];
    
    if (checkButtonHTTP.state == NSControlStateValueOn || ![settingPref isValidIP:hostHTTP])
        [settingPref unsetHTTP];
    else
        [settingPref setHttp:hostHTTP];
    
    if (checkButtonPHP.state == NSControlStateValueOff && !(argsPHP.length > 0) )
        [settingPref unsetPHP];
    else
        [settingPref setPhp:argsPHP];
}


- (IBAction)cancel:(id)sender {
    [self.prefwindow close];
}


- (IBAction)buttonHTTP:(NSButton *)sender {
    [serverHTTPField setEnabled:checkButtonHTTP.state != NSControlStateValueOn];
}


- (IBAction)buttonPHP:(NSButton *)sender {
    [argsPHPField setEnabled:checkButtonPHP.state == NSControlStateValueOn];
}

@end
