/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <arpa/inet.h>
#import "Setting.h"


@implementation Setting

@synthesize port;
@synthesize host;
@synthesize user;
@synthesize pass;
@synthesize http;
@synthesize php;

- (id) init {
    
   self = [super init];

    if ([self getPref:@"host"].length > 0)
        host = [self getPref:@"host"];
    else
        host = @"127.0.0.1";
    
    if ([self getPref:@"port"].length > 0) {
        int i = [[self getPref:@"port"] intValue];
        port = i;
    } else {
        port = 6600;
    }
    
    user = [self getPref:@"user"];
    pass = [self getPref:@"pass"];
    http = [self getPref:@"HTTP"];
    php = [self getPref:@"PHP"];
    
    return self;
}


- (void)setHost:(NSString *)aHost {
    [self setPref:aHost key:@"host"];
}


- (void)setPort:(unsigned int)aPort{
    [self setPref:[NSString stringWithFormat:@"%d",aPort] key:@"port"];
}

- (void)setUser:(NSString *)aUser {
    [self setPref:aUser key:@"user"];
}

- (void)unsetUser {
    user = nil;
    [self removePref:@"user"];
}

- (void)setPass:(NSString *)aPass {
    [self setPref:aPass key:@"pass"];
}

- (void)unsetPass {
    pass = nil;
    [self removePref:@"pass"];
}

- (void)setHttp:(NSString *)aHTTP {
    [self setPref:http key:@"HTTP"];
}

- (void)unsetHTTP {
    http = nil;
    [self removePref:@"HTTP"];
}

- (void)setPhp:(NSString *)aPHP {
    [self setPref:aPHP key:@"PHP"];
}

- (void)unsetPHP {
    php = nil;
    [self removePref:@"PHP"];
}


- (NSString *)getPref:(NSString *)key {
    NSUserDefaults *userPref = [NSUserDefaults standardUserDefaults];
    return [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:key] error:nil];
}

- (void)setPref:(NSString *)object key:(NSString *)key {
    NSUserDefaults *userPref = [NSUserDefaults standardUserDefaults];
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:object requiringSecureCoding:YES error:nil] forKey:key];
    [userPref synchronize];
}

- (void)removePref:(NSString *)key {
    NSUserDefaults *userPref = [NSUserDefaults standardUserDefaults];
    [userPref removeObjectForKey:key];
    [userPref synchronize];
}


- (BOOL)isValidIP:(NSString *)ipAddress  {
    int success;
    
    const char *bar = [ipAddress UTF8String];
    
    struct in_addr dst;
    success = inet_pton(AF_INET, bar, &dst);
    if (success != 1) {
        struct in6_addr dst6;
        success = inet_pton(AF_INET6, bar, &dst6);
    }
    
    return success == 1;
}


@end
