/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <UIImageView+WebCache.h>
#import "AlbumItem.h"
#import "AlbumBigCover.h"
#import "AlbumContentView.h"
#import "AlbumsView.h"
#import "MPDCache.h"
#import "MPDCommand.h"
#import "MPDQueue.h"
#import "MPDAlbum.h"
#import "Album.h"
#import "Setting.h"


@implementation AlbumItem

AlbumBigCover *bigCover;

int albumContentSize = 0;

MPDAlbum *mpdAlbum;

- (id) init {
    self = [super init];
    mpdAlbum = [[MPDAlbum alloc] init];
    return self;
}

- (void)setRepresentedObject:(id)representedObject{
    [super setRepresentedObject:representedObject];
    
    if (representedObject == nil) {
        
        [albumName setStringValue:@"No Value"];
        [artistName setStringValue:@"No Value"];
        
    } else {
        
        Setting *setting = [[Setting alloc] init];
        
        NSString *imageValue = [representedObject objectForKey:@"pathImage"];
        NSString *path = [NSString stringWithFormat:@"%@/%@", imageValue, @"cover.jpg"];
        NSString *web = [path stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet  URLPathAllowedCharacterSet]];
        NSString *http = setting.http;
        if (http == nil)
            http = setting.host;
        
        NSString *addPHP = [NSString stringWithFormat:@"%@/", setting.php];
        
        if (addPHP == nil)
            addPHP = @"";
        else
            web = [web stringByReplacingOccurrencesOfString:@"&" withString:@"%26"];
        
        NSString *finalurl = [NSString stringWithFormat:@"http://%@/%@%@", http, addPHP, web];
        NSURL *url = [NSURL URLWithString:finalurl];
        
        [albumName setStringValue:[representedObject objectForKey:@"albumName"]];
        [artistName setStringValue:[representedObject objectForKey:@"artistName"]];
        
        [albumCover sd_setImageWithURL:url placeholderImage:[UIImage imageNamed:@"no_cover"]];
        
        NSShadow *shadow = [[NSShadow alloc] init];
        [shadow setShadowBlurRadius:2.5f];
        [shadow setShadowOffset:CGSizeMake(1.5f, -2.0f)];
        
        NSColor *myColor = [NSColor colorWithRed:0 green:0 blue:0 alpha:0.6f];
        [shadow setShadowColor:myColor];
        [albumCover setShadow:shadow];
    }
}


- (IBAction)clickPlay:(NSButton *)sender {
    
    NSString *album = [[sender.target representedObject] objectForKey:@"albumName"];
    NSString *path = [[sender.target representedObject] objectForKey:@"pathImage"];
    [[MPDCommand sMPDCommand] playAlbum:album path:path position:-1];
}


- (IBAction)clickAlbum:(NSButton *)sender {
    NSString *album = [[sender.target representedObject] objectForKey:@"albumName"];
    NSString *artist = [[sender.target representedObject] objectForKey:@"artistName"];
    NSString *path = [[sender.target representedObject] objectForKey:@"pathImage"];
    
    Album *alb = [[Album alloc] init];
    [alb setAlbumName:album];
    [alb setArtist:artist];
    [alb setPath:path];
        
    NSArray *albumContentArray = [mpdAlbum getAlbumContent:alb];
    
    [[MPDCache sMPDCache] setAlbumContentArray:albumContentArray];
    
    albumContentSize = (int)[albumContentArray count];
    
    // évite le rechargement du contenu de l'ancien album
    // lors d'un changement de vue trop rapide
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 200 * NSEC_PER_MSEC), dispatch_get_main_queue(), ^{
        [self displayPopover:sender];
    });
}


- (IBAction)clickMaximize:(NSButton *)sender {
    NSString *imageValue = [[sender.target representedObject] objectForKey:@"pathImage"];
    NSString *path = [NSString stringWithFormat:@"%@/%@", imageValue, @"cover.jpg"];
    NSString *web = [path stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet  URLPathAllowedCharacterSet]];
    NSString* finalurl = [NSString stringWithFormat:@"http://127.0.0.1/%@", web];
    NSURL* url = [NSURL URLWithString:finalurl];
    
    bigCover = [[AlbumBigCover alloc] init:url];
    [bigCover showWindow:self];
}


- (IBAction)addToQueue:(NSMenuItem *)sender {
    
    NSString *album = [[sender.target representedObject] objectForKey:@"albumName"];
    NSString *artist = [[sender.target representedObject] objectForKey:@"artistName"];
    NSString *path = [[sender.target representedObject] objectForKey:@"pathImage"];
    
    Album *alb = [[Album alloc] init];
    [alb setAlbumName:album];
    [alb setArtist:artist];
    [alb setPath:path];
    
    NSArray *albumContentArray = [mpdAlbum getAlbumContent:alb];
    
    MPDQueue *mpdQueue = [[MPDQueue alloc] init];
    [mpdQueue addToQueue:[albumContentArray copy]];
}


-(void)displayPopover:(NSButton *)sender  {
    
    NSRect item = [sender.self convertRect: [sender.self bounds] toView: nil];
    
    AlbumContentView *contentAlbum = [[AlbumContentView alloc] init];
    NSViewController *popoverViewController = [contentAlbum initWithNibName: @"AlbumContent" bundle: nil];
    NSPopover *popover = [[NSPopover alloc] init];
    
    [popover setBehavior: NSPopoverBehaviorApplicationDefined];
    [popover setContentViewController: popoverViewController];
    [popover setContentSize: popoverViewController.view.frame.size];
    [popover showRelativeToRect: NSMakeRect(item.origin.x, item.origin.y, item.size.width , item.size.height) ofView: [[NSApp keyWindow] contentView] preferredEdge: NSMaxXEdge];
    
    int height = (118 + (19 * albumContentSize));
    
    if (height < 470)
        [popover setContentSize:CGSizeMake(580, height)];
    else
        [popover setContentSize:CGSizeMake(580, 470)];
    
    popover.behavior = NSPopoverBehaviorTransient;
}


@end
