/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "PlaylistView.h"
#import "CurrentPlaylisyView.h"
#import "MPDCommand.h"
#import "MPDPlayLists.h"
#import "Song.h"


@implementation PlaylistView

@synthesize tableOutletPlaylist = _tabOutletPlaylist;

NSTableView *tablePlaylist;
NSArray *playlistArray;
NSString *currentPl;

int currentPositionPlay;

MPDPlayLists *playLists;

- (id)init {
    self = [super init];
    playLists = [[MPDPlayLists alloc] init];
    return self;
}

- (void)awakeFromNib {
        
    currentPl = CurrentPlaylisyView.currentPlaylist;

    playlistArray = [playLists getPlaylist:currentPl];
    
    [_tabOutletPlaylist setTarget:self];
    [_tabOutletPlaylist setDoubleAction:@selector(doubleClick:)];
}

- (void)doubleClick:(id)object {
    
    unsigned int rowNumber = (unsigned int) [_tabOutletPlaylist clickedRow];
    
    [[MPDCommand sMPDCommand] playPlaylist:playlistArray position:rowNumber];
    
    [tablePlaylist reloadDataForRowIndexes:[NSIndexSet indexSetWithIndex:currentPositionPlay] columnIndexes:[NSIndexSet indexSetWithIndex:0]];
    
    [tablePlaylist reloadDataForRowIndexes:[NSIndexSet indexSetWithIndex:rowNumber] columnIndexes:[NSIndexSet indexSetWithIndex:0]];
    
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView {
    tablePlaylist = tableView;
    return [playlistArray count];
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    
    Song *song = playlistArray[row];
    
  //  MPDStatus *songPosition = mpdplaylist.getStatus;
    
    if ([tableColumn.identifier isEqualToString:@"trackNumber"]) {
        return [NSString stringWithFormat:@"%d",song.track];
    } else if ([tableColumn.identifier isEqualToString:@"artist"]) {
        return song.artist;
    } else if ([tableColumn.identifier isEqualToString:@"album"]) {
        return song.album;
    } else if ([tableColumn.identifier isEqualToString:@"title"]) {
        return song.title;
    } else if ([tableColumn.identifier isEqualToString:@"composer"]) {
        return song.composer;
    } else if ([tableColumn.identifier isEqualToString:@"time"]) {
        
        int elapsedMinutes = song.duration / 60;
        int elapsedSeconds = (int)song.duration % 60;
        NSString *elapsed = [NSString stringWithFormat:@"%2d:%02d",elapsedMinutes, elapsedSeconds];
        
        return elapsed;
    } else  if ([[tableColumn identifier] isEqualToString:@"isPlaying"]) {
        
      //  if (row == songPosition.titlePosition) {
       //     currentPositionPlay = (int)row;
            
        //    if (mpdplaylist.getState == MPD_STATE_PLAY )
         //       return [NSImage imageNamed:@"play_arrow_amber_a700_24pt"];
        //    else
        //        return [NSImage imageNamed:@"pause_amber_a700_24pt"];
      //  }
        
        return nil;
        
    } else {
        
        return nil;
    }
}

@end
