/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* *******************************************************************
 200 : succès de la requête ;
 301 et 302 : redirection, respectivement permanente et temporaire ;
 401 : utilisateur non authentifié ;
 403 : accès refusé ;
 404 : page non trouvée ;
 500 et 503 : erreur serveur ;
 504 : le serveur n'a pas répondu.
 ******************************************************************* */

#import "RadioLoader.h"
#import "Radio.h"
#import "NSString+HTML.h"


@implementation RadioLoader


- (id) init {
    self = [super init];
    return self;
}

- (NSArray *)getIndex {
    
    NSURL *url = [NSURL URLWithString:@"http://opml.radiotime.com/"];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    return [self radioList:resultat];
}


- (NSArray *)getListOf:(NSString *)aURL {
    
    NSURL *url = [NSURL URLWithString:aURL];
    NSData *data = [NSData dataWithContentsOfURL:url];    
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    if (resultat.length < 1 || [resultat containsString:@"403 - Forbidden: Access is denied"]) {
        
        Radio *radio = [[Radio alloc]init];
        [radio setLink:@"yes"];
        [radio setUrl:@"http://127.0.0.1"];
        [radio setName:@"403 - Forbidden: Access is denied"];
        
        return [NSArray arrayWithObject:radio];
    }
    
    return [self tri:resultat];
}


- (NSArray *)search:(NSString *)text {
    
    NSString *query = [text stringByReplacingOccurrencesOfString:@" " withString:@"%20"];
    query = [query stringByReplacingOccurrencesOfString: @"&" withString: @"&amp;"];
    query = [query stringByReplacingOccurrencesOfString: @"'" withString: @"%27"];
    
    NSString *recherche = [NSString stringWithFormat:@"http://opml.radiotime.com/Search.ashx?query=%@", query];
    NSURL *url = [NSURL URLWithString:recherche];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    return [self radioList:resultat];
}

- (NSArray *)radioList:(NSString *)string {
    
    if (string.length < 1 || [string containsString:@"403 - Forbidden: Access is denied"])
        return [NSArray arrayWithObject:[self error:string]];
        
    return [self tri:string];
}


- (NSArray *)tri:(NSString *)string {
    
    NSArray *array = [self regimeSec:string];
    NSMutableArray *radioList = [NSMutableArray array];
    
    for (NSString *string in array) {
        
        if ([string containsString:@"type=\"link\""])
            [radioList addObject:[self moulinette:string isFolder:YES]];
        else if ([string containsString:@"type=\"audio\""])
            [radioList addObject:[self moulinette:string isFolder:NO]];
    }
    
    return radioList;
}


- (Radio *)moulinette:(NSString *)string isFolder:(BOOL)isFolder {
    
    NSArray *array = [self decoupe:string];
    
    Radio *radio = [[Radio alloc]init];
    
    if (isFolder)
        [radio setLink:@"yes"];
    else
        [radio setLink:@"no"];
    
    for (NSString *str in array) {
        if ([str containsString:@"URL="])
            [radio setUrl:[self getString:str stringType:@"URL=\""]];
        
        if ([str hasPrefix:@"radioname="])
            [radio setName:[self getString:str stringType:@"radioname=\""]];
        
        if ([str containsString:@"subtext="] && !isFolder)
            [radio setInfo:[self getString:str stringType:@"subtext=\""]];
        
        if ([str containsString:@"image="] && !isFolder)
            [radio setLogoUrl:[self getString:str stringType:@"image=\""]];
        
    }
    
    return radio;
    
}

- (NSArray *)decoupe:(NSString *)string {
    
    NSString *str = [string stringByReplacingOccurrencesOfString:@"<outline " withString:@""];
    str = [str stringByReplacingOccurrencesOfString:@"/>" withString:@""];
    str = [str stringByReplacingOccurrencesOfString:@"\" " withString:@"\"\n"];
    
    return [str componentsSeparatedByString:@"\n"];
}

- (NSArray *)regimeSec:(NSString *)data {
    
    NSString *slim = data;
    
    slim = [slim stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
    
    // evite les bug avec subtext
    slim = [slim stringByReplacingOccurrencesOfString:@"\" text=\"" withString:@"\" radioname=\""];
    
    if ([slim containsString:@"<status>200</status>"]) {
        
        NSRange range = [slim rangeOfString:@"<outline"];
        slim = [slim substringFromIndex:range.location];
        NSRange range2 = [slim rangeOfString:@"</body>"];
        slim = [slim substringToIndex:range2.location];
        
        return [slim componentsSeparatedByString:@"\n"];
    }
    
    return nil;
}


- (NSString *)getString:(NSString *)aLine stringType:(NSString *)stringType {
    
    if ([stringType isEqualToString:@"radioname=\""] || [stringType isEqualToString:@"subtext=\""])
        aLine = aLine.htmlDecode;
    
    NSRange r1 = [aLine rangeOfString:stringType];
    NSString *name = [aLine substringFromIndex:(r1.location + r1.length)];
    NSRange r2 = [name rangeOfString:@"\""];
    NSString *final = [name substringToIndex:((r2.location-1) + r2.length)];
    
    if (final == (id)[NSNull null] || final.length == 0) {
        
        if ([stringType isEqualToString:@"URL=\""] )
            final = @"null0";
        else if ([stringType isEqualToString:@"radioname=\""])
            final = @"null1";
    }
    
    return final;
}


- (Radio *)error:(NSString *)string {
    Radio *radio = [[Radio alloc]init];
    
    [radio setLink:@"yes"];
    [radio setUrl:@"0"];
    
    if (string.length > 1 || [string containsString:@"403 - Forbidden: Access is denied"])
        [radio setName:@"403 - Forbidden: Access is denied"];
    else
        [radio setName:@"Erreur de connexion"];
    
    return radio;
    
}

@end
