/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "StreamEdit.h"
#import "MPDCache.h"
#import "MPDStream.h"


@implementation StreamEdit

Stream *streamToEdit;

unsigned int streamEditPosition;

BOOL addRadio = NO;

- (instancetype)init:(Stream *)stream {
    self = [super initWithWindowNibName:@"StreamEdit"];
    
    streamToEdit = stream;
    // toujours au premier plan
    [self.window makeKeyAndOrderFront:nil];
    [self.window setLevel:NSStatusWindowLevel];
    
    return self;
}


- (void)awakeFromNib {
    
    if (streamToEdit == nil) {
        addRadio = YES;

    } else {
        
        [nameField setStringValue:streamToEdit.name];
        [urlField setStringValue:streamToEdit.url];
        
        streamEditPosition = streamToEdit.position;
    }
}


- (IBAction)save:(id)sender {
    
    NSString *url = [urlField stringValue];
    NSString *name = [nameField stringValue];
    
    MPDStream *mpdStream = [[MPDStream alloc] init];
    
    if (!addRadio)
        [mpdStream deleteStream:streamEditPosition];
    
    if (url.length > 7 && name.length > 0 && [[url substringToIndex:4] isEqualToString:@"http"]) {
        [mpdStream addStream:[NSString stringWithFormat:@"%@#%@", url, name]];
        
        [[MPDCache sMPDCache]streamUpdateTrigger];

        [self close];
        
    } else {
        
        if (name.length > 0)
            labelName.textColor = [NSColor labelColor];
        else
            labelName.textColor = errorColor;
        
        if ( url.length > 7 && [ [url substringToIndex:4] isEqualToString:@"http"] )
            labelURL.textColor = [NSColor labelColor];
         else
            labelURL.textColor = errorColor;
    }
    
    [self boxClosed];
}


- (IBAction)cancel:(id)sender {
    [self close];
    [self boxClosed];
}

- (void)boxClosed {
        [[NSNotificationCenter defaultCenter] postNotificationName:@"editStreamBoxClose" object:nil];
}


@end
