/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "StreamView.h"
#import "StreamEdit.h"
#import "ConfirmDialog.h"
#import "MPDCommand.h"
#import "MPDStream.h"
#import "Stream.h"


@implementation StreamView

@synthesize streamTableView = _streamTableView;

StreamEdit *streamEdit;
MPDStream *mpdStream;

// Ne pas inclure dans la méthode sinon IBAction ne fonctionne pas
ConfirmDialog *confirmStream;

NSArray *streamArray;
NSMutableArray *selectedItems;

long rowStreamSelected;

BOOL editActive = NO;

- (id)init {
    self = [super init];
    mpdStream = [[MPDStream alloc] init];
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [MPDCache sMPDCache].delegateStream = self;
    [self editBoxListener];
    
    [self deleteBoxListener];
    
    [_streamTableView setTarget:self];
    [_streamTableView setDoubleAction:@selector(doubleClick:)];
}

- (void)viewDidAppear {

}

- (void)awakeFromNib {
    
    streamArray =  mpdStream.getStreamList;
    streamArray = [self sortList:streamArray];
    
    [self performSelectorOnMainThread:@selector(updateTable:) withObject:_streamTableView waitUntilDone:NO];
}


- (void)doubleClick:(id)object {
    
    int rowNumber = (int) [_streamTableView clickedRow];
    Stream *stream = streamArray[rowNumber];
    [[MPDCommand sMPDCommand] playStream:stream.url];
}

- (void)updateTable:(NSTableView *)table {
    [table reloadData];
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView {
    return [streamArray count];
}


- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    
    Stream *stream = streamArray[row];
    
    NSTableCellView *cell = [tableView makeViewWithIdentifier:@"name" owner:self];
    
    if ([tableColumn.identifier isEqualToString:@"name"])
        return cell.textField.stringValue = stream.name;
    else
        return nil;
}


- (IBAction)edit:(NSMenuItem *)sender {
    
    if (editActive)
        return;
    
    Stream *stream = streamArray[rowStreamSelected];
    streamEdit = [[StreamEdit alloc] init:stream];
    [streamEdit showWindow:self];
    editActive = YES;
    
}


- (IBAction)delete:(NSMenuItem *)sender {
    
    if (editActive)
        return;
        
    NSIndexSet *row = [_streamTableView selectedRowIndexes];
    NSMutableArray *selection = [NSMutableArray array];
    [row enumerateIndexesUsingBlock:^(NSUInteger idx, BOOL *stop) {
        [selection addObject:[NSNumber numberWithInteger:idx]];
    }];
        
    selectedItems = [NSMutableArray array];
        
    // Prend en compte la/les lignes en subrillance si > 1 sinon prend celle sous le click droit.
    if ([selection count] > 1) {
            
        NSArray *realList = mpdStream.getStreamList;
            
        for (NSString *string in selection) {
                
            int num = [string intValue];
            Stream *current = streamArray[num];
                
            for (Stream *real in realList) {
                    
                if ([current.url isEqualToString:real.url] && [current.url isEqualToString:real.url])
                    [selectedItems addObject:[NSString stringWithFormat:@"%d", real.position]];
            }
        }
            
        selectedItems = [[selectedItems sortedArrayUsingSelector: @selector(compare:)] copy];
    } else {
            
        Stream *current = streamArray[rowStreamSelected];
            
        [selectedItems addObject:[NSString stringWithFormat:@"%d", current.position] ];
    }
        
    NSString *text;
        
    if ([selectedItems count] > 1) {
        text = [NSString stringWithFormat:@"Confirmer la suppression des %d éléments", (int)[selectedItems count]];
            
    } else {
        Stream *stream = streamArray[rowStreamSelected];
        text = [NSString stringWithFormat:@"Confirmer la suppression de: %@", stream.name];
    }
        
    NSMutableArray *marray = [NSMutableArray array];
    [marray addObject:@"Supprimer stream"];
    [marray addObject:text];
    [marray addObject:@"deleteStreamBoxClose"];
        
    confirmStream = [[ConfirmDialog alloc] init:marray];
    [confirmStream showWindow:self];
        
    editActive = YES;
}

- (void)menuWillOpen:(NSMenu *)menu {
    rowStreamSelected = (long)_streamTableView.clickedRow;
}


- (void)onStreamChange {
    streamArray =  mpdStream.getStreamList;
    streamArray = [self sortList:streamArray];
    [_streamTableView reloadData];
}


- (NSArray *)sortList:(NSArray *)aMArray {
    
    NSArray *sortedArray = [aMArray sortedArrayUsingComparator:^(Stream* lhn, Stream* rhn) {
        NSString *leftStream = lhn.name;
        NSString *rightStream = rhn.name;
        
        return [leftStream compare:rightStream options:NSCaseInsensitiveSearch];
    }];
    
    return sortedArray;
}

- (void)editBoxListener {
    
    [[NSNotificationCenter defaultCenter] addObserverForName:@"editStreamBoxClose" object:nil queue:nil usingBlock:^(NSNotification *note) {
        
        editActive = NO;
        
    }];
}

- (void)deleteBoxListener {
    
    [[NSNotificationCenter defaultCenter] addObserverForName:@"deleteStreamBoxClose" object:nil queue:nil usingBlock:^(NSNotification *note) {
        
        NSDictionary* userInfo = note.userInfo;
        NSString* what = (NSString *)userInfo[@"close"];
        
        if ([what isEqualToString:@"delete"]) {

            unsigned int i = 0;
            for (NSString *num in selectedItems) {
                [mpdStream deleteStream:[num intValue] - i];
                i++;
            }
            
            streamArray =  mpdStream.getStreamList;
            streamArray = [self sortList:streamArray];
            [self->_streamTableView reloadData];
        }
        
        editActive = NO;
        
    }];
}

@end
