/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "ProgressSlider.h"
#import "ProgressSliderCell.h"


@interface NSImage (Rotated)
- (NSImage *)imageRotated:(CGFloat)degrees;
@end


@implementation ProgressSlider
//  We need to override it to prevent drawing bugs
//  Follow this link to know more about it:
//  http://stackoverflow.com/questions/3985816/custom-nsslidercell
- (void)setNeedsDisplayInRect:(NSRect)invalidRect {
    [super setNeedsDisplayInRect:[self bounds]];
}

- (void)awakeFromNib {
    [super awakeFromNib];
    
    if(![self.cell isKindOfClass:[ProgressSliderCell class]]) {
        self.cell = [[ProgressSliderCell alloc] init];
    }
}

- (id)initWithKnobImage:(NSImage *)knob {
    self = [super init];
    if (self) {
        self.cell = [[ProgressSliderCell alloc] initWithKnobImage:knob];
        return !self.cell ? nil : self;
    }

    return self;
}

- (id)initWithKnobImage:(NSImage *)knob minimumValueImage:(NSImage *)minImage maximumValueImage:(NSImage *)maxImage {
    self = [super init];
    if (self) {
        self.cell = [[ProgressSliderCell alloc] initWithKnobImage:knob minimumValueImage:minImage maximumValueImage:maxImage];
        return !self.cell ? nil : self;
    }

    return self;
}

- (ProgressSliderCell *)sliderCell {
	return self.cell;
}

- (NSImage *)knobImage {
    return self.sliderCell.knobImage;
}

- (void)setKnobImage:(NSImage *)image {
    self.sliderCell.knobImage = image;
}

- (NSImage *)minimumValueImage {
    return self.sliderCell.minimumValueImage;
}

- (void)setMinimumValueImage:(NSImage *)minimumValueImage {
    if (NSEdgeInsetsEqual(minimumValueImage.capInsets, NSEdgeInsetsZero)) {
        CGFloat inset = minimumValueImage.size.width-1;
        if (self.sliderCell.vertical) {
            minimumValueImage = [minimumValueImage imageRotated:90];
            minimumValueImage.capInsets = NSEdgeInsetsMake(0, 0, inset, 0);
        } else {
            minimumValueImage.capInsets = NSEdgeInsetsMake(0, inset, 0, 0);
        }
	}
	self.sliderCell.minimumValueImage = minimumValueImage;
}

- (NSImage *)maximumValueImage {
    return self.sliderCell.maximumValueImage;
}

- (void)setMaximumValueImage:(NSImage *)maximumValueImage {
    if (NSEdgeInsetsEqual(maximumValueImage.capInsets, NSEdgeInsetsZero)) {
        CGFloat inset = maximumValueImage.size.width-1;
        if (self.sliderCell.vertical) {
            maximumValueImage = [maximumValueImage imageRotated:90];
            maximumValueImage.capInsets = NSEdgeInsetsMake(inset, 0, 0, 0);
        } else {
            maximumValueImage.capInsets = NSEdgeInsetsMake(0, 0, 0, inset);
        }
	}
	self.sliderCell.maximumValueImage = maximumValueImage;
}

@end

@implementation NSImage (Rotated)

// Source from https://gist.github.com/Rm1210/10621763
- (NSImage *)imageRotated:(CGFloat)degrees {
    degrees = fmod(degrees, 360.);
    if (0 == degrees) {
        return self;
    }
    NSSize size = [self size];
    NSSize maxSize;
    if (90. == degrees || 270. == degrees || -90. == degrees || -270. == degrees) {
        maxSize = NSMakeSize(size.height, size.width);
    } else if (180. == degrees || -180. == degrees) {
        maxSize = size;
    } else {
        maxSize = NSMakeSize(20+MAX(size.width, size.height), 20+MAX(size.width, size.height));
    }
    NSAffineTransform *rot = [NSAffineTransform transform];
    [rot rotateByDegrees:degrees];
    NSAffineTransform *center = [NSAffineTransform transform];
    [center translateXBy:maxSize.width / 2. yBy:maxSize.height / 2.];
    [rot appendTransform:center];
    NSImage *image = [[NSImage alloc] initWithSize:maxSize];
    [image lockFocus];
    [rot concat];
    NSRect rect = NSMakeRect(0, 0, size.width, size.height);
    NSPoint corner = NSMakePoint(-size.width / 2., -size.height / 2.);
    [self drawAtPoint:corner fromRect:rect operation:NSCompositingOperationCopy fraction:1.0];
    [image unlockFocus];
    return image;
}

@end
